# -*- coding: utf-8 -*-
import hashlib
import json

from typing import Dict

from django import template
from django.template import Node
from django.utils.safestring import mark_safe

from django_svelte_married.settings import (SVELTE_RELOADING_ENABLED, SVELTE_RELOADING_PORT, SVELTE_RENDERER)

register = template.Library()


def _hash_component_props(component: str, props: any = None, idx: int = 0) -> str:
    """Creates unique hash of component and props to uniquely identify component in `SVELTE_COMPONENTS`"""
    component_props_str = json.dumps([component, props, idx])
    return hashlib.md5(component_props_str.encode('utf-8')).hexdigest()


class SvelteNode(Node):
    def __init__(self, nodelist, args):
        self.nodelist = nodelist
        self.args = args
        super().__init__()

    def render(self, context):
        if 'SVELTE_COMPONENTS' not in context:
            raise Exception("You need to add 'django_svelte_married.context_processors.svelte' as a context processor "
                            "in `TEMPLATES` of your project's settings.py")
        if len(self.args) < 2:
            raise Exception("tbd")
        component, props = self.args[0].resolve(context), self.args[1].resolve(context)
        props_jsons = json.loads(props) if isinstance(props, str) else props
        mount_id = _hash_component_props(component, props_jsons, len(context['SVELTE_COMPONENTS'].keys()))
        context['SVELTE_COMPONENTS'][mount_id] = {'component': component, 'props': props_jsons}
        context.update(dict(id=mount_id, content=mark_safe(self.render_ssr(component, props))))
        return self.nodelist.render(context).strip()

    def render_ssr(self, component, props):
        return SVELTE_RENDERER(component=component, props=props)


@register.tag("svelte")
def svelte(parser, token):
    args = [parser.compile_filter(t) for t in token.split_contents()[1:]]
    nodelist = parser.parse(('endsvelte',))
    parser.delete_first_token()
    return SvelteNode(nodelist, args)

# def svelte(context: Dict[str, any], component: str, props: any = None):
# if 'SVELTE_COMPONENTS' not in context:
#     context['SVELTE_COMPONENTS'] = {}

# context['SVELTE_COMPONENTS'][mount_id] = {'component': component, 'props': props_jsons}
# return mark_safe(f"id=\"{mount_id}\"")


@register.inclusion_tag('svelte_print.html', takes_context=True)
def svelte_print(context: Dict[str, any]):
    return {'SVELTE_PRINT': mark_safe(json.dumps(context.get('SVELTE_COMPONENTS', {}))),
            'SVELTE_LIVERELOAD_ENABLED': SVELTE_RELOADING_ENABLED,
            'SVELTE_LIVERELOAD_PORT': SVELTE_RELOADING_PORT}



import os
from subprocess import run
from sys import stdout, stderr

from django.core.management import BaseCommand

from django_svelte_married.settings import SVELTE_PROJECT_PATH, SVELTE_RELOADING_PORT


class Command(BaseCommand):
    def handle(self, *args, **options):
        # copy necessary environment from `settings.py` to be used in Node
        os.environ["SVELTE_RELOADING_PORT"] = str(SVELTE_RELOADING_PORT)
        # run npm that handles parallel procs
        run("npm run dev", cwd=SVELTE_PROJECT_PATH, stdout=stdout, stderr=stderr, shell=True)



import json
import os
import shutil
from sys import stdout, stderr
from subprocess import run

from django.core.management import BaseCommand
from django_svelte_married.settings import SVELTE_PROJECT_PATH
from django.conf import settings

SCRIPT_DIR = os.path.dirname(__file__)


class Command(BaseCommand):
    def handle(self, *args, **options):
        self.stdout.write(f"Install `django_svelte_married` runtime npm dependencies")
        with open(os.path.join(SVELTE_PROJECT_PATH, 'package.json'), 'r') as pkg_f:
            pkg_json = json.load(pkg_f)
            managepy_rel_path = os.path.relpath(settings.BASE_DIR, SVELTE_PROJECT_PATH)

            pkg_json['dependencies']['npm-run-all'] = "*"
            pkg_json['dependencies']['cross-env'] = "*"
            pkg_json['dependencies']['express'] = "*"
            pkg_json['dependencies']['body-parser'] = "*"

            if 'dev' in pkg_json['scripts'] and 'run-p' in pkg_json['scripts']['dev']:
                self.stdout.write("Looks like `package.json` is correctly provisioned.")
            else:
                pkg_json['scripts']['dev'] = "run-p -l dev:django dev:dom dev:ssr dev:renderer"
                pkg_json['scripts']['dev:django'] = f"cross-env python {managepy_rel_path}/manage.py runserver"
                pkg_json['scripts']['dev:dom'] = "cross-env BUILD_TYPE=DOM rollup -c -w"
                pkg_json['scripts']['dev:ssr'] = "cross-env BUILD_TYPE=RENDERER rollup ./src/exported.js -c -w"
                pkg_json['scripts']['dev:renderer'] = "cross-env node ./scripts/renderer.js"

            os.makedirs(os.path.join(SVELTE_PROJECT_PATH, "scripts"), exist_ok=True)
            os.makedirs(os.path.join(SVELTE_PROJECT_PATH, "src"), exist_ok=True)

            shutil.copy(os.path.join(SCRIPT_DIR, '../../js/renderer.js'),
                        os.path.join(SVELTE_PROJECT_PATH, "scripts/renderer.js"))
            shutil.copy(os.path.join(SCRIPT_DIR, '../../js/main.js'),
                        os.path.join(SVELTE_PROJECT_PATH, "src/main.js"))
            if not os.path.exists(os.path.join(SVELTE_PROJECT_PATH, "src/exported.js")):
                shutil.copy(os.path.join(SCRIPT_DIR, '../../js/exported.js'),
                            os.path.join(SVELTE_PROJECT_PATH, "src/exported.js"))

        with open(os.path.join(SVELTE_PROJECT_PATH, 'package.json'), 'w') as pkg_f:
            pkg_f.write(json.dumps(pkg_json, indent=4))

        self.stdout.write(f"Install npm dependencies in `{SVELTE_PROJECT_PATH}`...")
        run("npm install", cwd=SVELTE_PROJECT_PATH, stdout=stdout, stderr=stderr, shell=True)

        self.stdout.write("")
        self.stdout.write("")
        self.stdout.write("Congratulations. You can now run `python manage.py runsvelte`")



import os
from django.conf import settings
from django.utils.module_loading import import_string

SVELTE_PROJECT_PATH = getattr(settings, 'SVELTE_PROJECT_PATH', os.path.join(settings.BASE_DIR, 'svelte_project'))

SVELTE_RELOADING_ENABLED = getattr(settings, 'SVELTE_RELOADING_ENABLED', True)
SVELTE_RELOADING_PORT = getattr(settings, 'SVELTE_RELOADING_PORT', 5050)

SVELTE_RENDERER_ENABLED = getattr(settings, 'SVELTE_RENDERER_ENABLED', True)
SVELTE_RENDERER_PORT = getattr(settings, 'SVELTE_RENDERER_PORT', 6060)
SVELTE_RENDERER = import_string(getattr(settings, 'SVELTE_RENDERER', 'django_svelte_married.renderers.local_renderer'))



import json

import requests
from django.utils.log import request_logger
from requests.exceptions import ConnectionError

from .settings import (SVELTE_RENDERER_PORT, SVELTE_RENDERER_ENABLED)


def local_renderer(component: str, props: any = None) -> str:
    if not SVELTE_RENDERER_ENABLED:
        return ""
    try:
        if not isinstance(props, str):
            props = json.dumps(props)
        resp = requests.post(f"http://localhost:{SVELTE_RENDERER_PORT}/", json=dict(component=component, props=props))
        if resp.status_code >= 300:
            request_logger.warn("Renderer backend responded with incorrect status code.")
            return ""
        try:
            return resp.json()['html']
        except Exception as e:
            request_logger.error("Renderer backend returned malformed, likely non-json response.")
            request_logger.error(e)
    except ConnectionError:
        request_logger.warn("Error when connecting to renderer backend.")
    return ""



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_svelte_married',
 'django_svelte_married.management',
 'django_svelte_married.management.commands',
 'django_svelte_married.templatetags',
 'django_svelte_married.tests']

package_data = \
{'': ['*'], 'django_svelte_married': ['js/*', 'static/svelte/*', 'templates/*']}

install_requires = \
['requests>=2.26.0,<3.0.0']

entry_points = \
{'console_scripts': ['test = scripts:test']}

setup_kwargs = {
    'name': 'django-svelte-married',
    'version': '0.0.4',
    'description': 'JGM stands for "Just-Got-Married", because Django and Svelte is a perfect match 🤵👰.',
    'long_description': None,
    'author': 'niespodd',
    'author_email': 'dariusz@ticketwhat.pl',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7',
}


setup(**setup_kwargs)
